@@ -1,7 +1,7 @@
 ############## last change 2012/03/16 #####################
 
 .Contlcmm <-
-    function(fixed,mixture,random,subject,classmb,ng,idiag,nwg,cor,data,B,convB,convL,convG,prior,maxiter,epsY,idlink0,ntrtot0,nbzitr0,zitr,nsim,call,Ydiscrete,subset,na.action,posfix,partialH,verbose,returndata,var.time){
+    function(fixed,mixture,random,subject,classmb,ng,idiag,nwg,cor,data,B,convB,convL,convG,prior,maxiter,epsY,idlink0,ntrtot0,nbzitr0,zitr,nsim,call,Ydiscrete,subset,na.action,posfix,partialH,verbose,returndata,var.time,nproc,clustertype){
         
         cl <- match.call()
 
@@ -21,10 +21,10 @@
         if(ng==1&nwg==TRUE) stop ("The argument nwg should be FALSE for ng=1")
 
 
-        if(class(fixed)!="formula") stop("The argument fixed must be a formula")
-        if(class(mixture)!="formula") stop("The argument mixture must be a formula")
-        if(class(random)!="formula") stop("The argument random must be a formula")
-        if(class(classmb)!="formula") stop("The argument classmb must be a formula")
+        if(!inherits(fixed,"formula")) stop("The argument fixed must be a formula")
+        if(!inherits(mixture,"formula")) stop("The argument mixture must be a formula")
+        if(!inherits(random,"formula")) stop("The argument random must be a formula")
+        if(!inherits(classmb,"formula")) stop("The argument classmb must be a formula")
         if(missing(data)){ stop("The argument data should be specified and defined as a data.frame")} 
         if(missing(subject)){ stop("The argument subject must be specified in any model even without random-effects")} 
         if(!is.numeric(data[[subject]])) stop("The argument subject must be numeric")
@@ -464,9 +464,12 @@
         pbH0 <- rep(0,NPM)
         if(is.logical(partialH))
         {
-            if(idlink0==2) pbH0[NPROB+NEF+NVC+NW+1:ntrtot0] <- 1
-            pbH0[posfix] <- 0
-            if(sum(pbH0)==0 & Hr0==1) stop("No partial Hessian matrix can be defined")
+            if(isTRUE(partialH))
+            {
+                if(idlink0==2) pbH0[NPROB+NEF+NVC+NW+1:ntrtot0] <- 1
+                pbH0[posfix] <- 0
+                if(sum(pbH0)==0 & Hr0==1) stop("No partial Hessian matrix can be defined")
+            }
         }
         else
         {
@@ -474,120 +477,16 @@
             pbH0[Hr0] <- 1
             pbH0[posfix] <- 0
         }
+        indexHr <- NULL
+        if(sum(pbH0)>0) indexHr <- which(pbH0==1)
 
 
 
         if(missing(B)){
-
-            if(ng0>1){
-                idea2 <- idea0
-                idprob2 <- rep(0,nv0)  
-                idg2 <- rep(0,nv0) 
-                idg2[idg0!=0] <- 1
-                NEF2 <- sum(idg2==1)-1
-                NPM2 <- NEF2+NVC+ntrtot0+ncor0
-                nwg2 <- 0
-                ng2 <- 1
-                ppi2 <- rep(0,ns0)
-                pred_m_g2 <- rep(0,nobs0)
-                pred_ss_g2 <- rep(0,nobs0)
-                maxiter2 <- min(75,maxiter)
-                convB2 <- max(0.01,convB)
-                convL2 <- max(0.01,convL)
-                convG2 <- max(0.01,convG)
-                Hr02 <- 0
-
-                V2 <- rep(0,NPM2*(NPM2+1)/2)
-
-                marker <- rep(0,nsim)
-                transfY <- rep(0,nsim)
-
-                                        
-
-                init <- .Fortran(C_hetmixcont,
-                                 as.double(Y0),
-                                 as.double(X0),
-                                 as.integer(prior2),
-                                 as.integer(idprob2),
-                                 as.integer(idea2),
-                                 as.integer(idg2),
-                                 as.integer(idcor0),
-                                 as.integer(ns0),
-                                 as.integer(ng2),
-                                 as.integer(nv0),
-                                 as.integer(nobs0),
-                                 as.integer(nea0),
-                                 as.integer(nmes0),
-                                 as.integer(idiag0),
-                                 as.integer(nwg2),
-                                 as.integer(ncor0),
-                                 as.integer(NPM2),
-                                 best=as.double(b1),
-                                 V=as.double(V2),
-                                 as.double(loglik),
-                                 niter=as.integer(ni),
-                                 conv=as.integer(istop),
-                                 as.double(gconv),
-                                 as.double(ppi2),
-                                 as.double(resid_m),
-                                 as.double(resid_ss),
-                                 as.double(pred_m_g2),
-                                 as.double(pred_ss_g2),
-                                 predRE=as.double(predRE),
-                                 as.double(convB2),
-                                 as.double(convL2),
-                                 as.double(convG2),
-                                 as.integer(maxiter2),
-                                 as.double(epsY),
-                                 as.integer(idlink0),
-                                 as.integer(nbzitr0),
-                                 as.double(zitr),
-                                 as.double(marker),
-                                 as.double(transfY),
-                                 as.integer(nsim),
-                                 as.double(Yobs),
-                                 as.integer(Ydiscrete),
-                                 as.double(vraisdiscret),
-                                 as.double(UACV),
-                                 as.double(rlindiv),
-                                 as.integer(pbH0),
-                                 as.integer(fix0))
-
-
-                k <- NPROB
-                l <- 0
-                t<- 0
-                for (i in 1:nvar.exp)    {
-                    if(idg0[i]==1 & i>1){
-                        l <- l+1
-                        t <- t+1
-                        b[k+t] <- init$best[l]
-                    }
-                    if(idg0[i]==2){
-                        if (i==1){
-                            for (g in 2:ng){
-                                t <- t+1
-                                b[k+t] <- -0.5*(g-1)
-                            }
-                        }
-                        if (i>1){
-
-                            l <- l+1
-                            for (g in 1:ng){
-                                t <- t+1
-                                if(init$conv==1) b[k+t] <- init$best[l]+(g-(ng+1)/2)*sqrt(init$V[l*(l+1)/2])
-                                else b[k+t] <- init$best[l]+(g-(ng+1)/2)*init$best[l]
-                            }
-                        }
-                    }
-                }
-
-                b[(NPROB+NEF+1):(NPROB+NEF+NVC)] <- init$best[(NEF2+1):(NEF2+NVC)]
-                b[(NPM-ntrtot0-ncor0+1):(NPM-ncor0)] <- init$best[(NPM2-ntrtot0-ncor0+1):(NPM2-ncor0)]
-                if (ncor0>0) {b[(NPROB+NEF+NVC+NW+ntrtot0+1):(NPROB+NEF+NVC+NW+ntrtot0+ncor0)] <- init$best[(NPM2-ncor0+1):NPM2]}
-            } 
             if(ng0==1 ){
                 b <- b1
+            } else {
+                stop("Please specify initial values with argument 'B'")
             }
         } 
         else
@@ -606,7 +505,7 @@
                             {
                                 ## remplacer varcov des EA par les prm a estimer
                                 
-                                if(idiag==1)
+                                if(idiag0==1)
                                 {
                                     b[NPROB+NEF+1:NVC] <- sqrt(b[NPROB+NEF+1:NVC])
                                 }
@@ -627,7 +526,7 @@
                     }
                 else
                     {
-                         if(class(B)!="lcmm") stop("B should be either a vector or an object of class lcmm")
+                         if(!inherits(B,"lcmm")) stop("B should be either a vector or an object of class lcmm")
                          
                          if(ng>1 & B$ng==1)
                             {
@@ -876,6 +775,7 @@
         if(NW!=0)names(b)[(NPROB+NEF+NVC+1):(NPROB+NEF+NVC+NW)] <- paste("varprop class",c(1:(ng0-1)))
 
         if (ncor0>0) {names(b)[(NPROB+NEF+NVC+NW+ntrtot0+1):(NPROB+NEF+NVC+NW+ntrtot0+ncor0)] <- paste("cor",1:ncor0,sep="")}
+        names_best <- names(b)
 
         N <- NULL
         N[1] <- NPROB
@@ -892,109 +792,198 @@
                                         #cat("valeurs initiales : ",b,"\n")
 ################ Sortie ###########################
         ptm <- proc.time()
-        if(verbose==TRUE) cat("Be patient, lcmm is running ... \n")
 
 
         marker <- rep(0,nsim)
         transfY <- rep(0,nsim)
 
 
-        out <- .Fortran(C_hetmixcont,
-                        as.double(Y0),
-                        as.double(X0),
-                        as.integer(prior0),
-                        as.integer(idprob0),
-                        as.integer(idea0),
-                        as.integer(idg0),
-                        as.integer(idcor0),
-                        as.integer(ns0),
-                        as.integer(ng0),
-                        as.integer(nv0),
-                        as.integer(nobs0),
-                        as.integer(nea0),
-                        as.integer(nmes0),
-                        as.integer(idiag0),
-                        as.integer(nwg0),
-                        as.integer(ncor0),
-                        as.integer(NPM),
-                        best=as.double(b),
-                        V=as.double(V),
-                        loglik=as.double(loglik),
-                        niter=as.integer(ni),
-                        conv=as.integer(istop),
-                        gconv=as.double(gconv),
-                        ppi2=as.double(ppi0),
-                        resid_m=as.double(resid_m),
-                        resid_ss=as.double(resid_ss),
-                        pred_m_g=as.double(pred_m_g),
-                        pred_ss_g=as.double(pred_ss_g),
-                        predRE=as.double(predRE),
-                        as.double(convB),
-                        as.double(convL),
-                        as.double(convG),
-                        as.integer(maxiter),
-                        as.double(epsY),
-                        as.integer(idlink0),
-                        as.integer(nbzitr0),
-                        as.double(zitr),
-                        marker=as.double(marker),
-                        transfY=as.double(transfY),
-                        as.integer(nsim),
-                        Yobs=as.double(Yobs),
-                        as.integer(Ydiscrete),
-                        vraisdiscret=as.double(vraisdiscret),
-                        UACV=as.double(UACV),
-                        rlindiv=as.double(rlindiv),
-                        as.integer(pbH0),
-                        as.integer(fix0))
+            
+        nfix <- sum(fix0)
+        bfix <- 0
+        if(nfix>0)
+        {
+            bfix <- b[which(fix0==1)]
+            b <- b[which(fix0==0)]
+            NPM <- NPM-nfix
+        }
+        
+        minY0 <- 0
+        maxY0 <- 0
+        ide0 <- 0
+        
+        if(maxiter==0)
+        {
+            vrais <- logliklcmm(b,Y0,X0,prior0,idprob0,idea0,idg0,idcor0,
+                                ns0,ng0,nv0,nobs0,nea0,nmes0,idiag0,nwg0,ncor0,
+                                NPM,epsY,idlink0,nbzitr0,zitr,minY0,maxY0,ide0,
+                                fix0,nfix,bfix)
+            
+            out <- list(conv=2, V=rep(0, length(b)), best=b,
+                        ppi2=rep(NA,ns0*ng0), predRE=rep(NA,ns0*nea0),
+                        resid_m=rep(NA,nobs0), resid_ss=rep(NA,nobs0),
+                        marker=rep(NA,nsim), transfY=rep(NA,nsim),
+                        Yobs=rep(NA,nobs0), 
+                        pred_m_g=rep(NA,nobs0*ng0), pred_ss_g=rep(NA,nobs0*ng0),
+                        vraisdiscret=NA, UACV=NA, rlindiv=rep(NA,ns0),
+                        gconv=rep(NA,3), niter=0, loglik=vrais)
+        }
+        else
+        {   
+            res <- mla(b=b, m=length(b), fn=logliklcmm,
+                       clustertype=clustertype,.packages=NULL,
+                       epsa=convB,epsb=convL,epsd=convG,partialH=indexHr,
+                       digits=8,print.info=verbose,blinding=FALSE,
+                       multipleTry=25,file="",
+                       nproc=nproc,maxiter=maxiter,minimize=FALSE,
+                       Y0=Y0,X0=X0,prior0=prior0,idprob0=idprob0,idea0=idea0,
+                       idg0=idg0,idcor0=idcor0,ns0=ns0,ng0=ng0,nv0=nv0,nobs=nobs0,
+                       nea0=nea0,nmes0=nmes0,idiag=idiag0,nwg0=nwg0,ncor0=ncor0,
+                       npm0=NPM,epsY0=epsY,idlink0=idlink0,nbzitr0=nbzitr0,zitr0=zitr,
+                       minY0=minY0,maxY0=maxY0,ide0=ide0,
+                       fix0=fix0,nfix0=nfix,bfix0=bfix)
+            
+            out <- list(conv=res$istop, V=res$v, best=res$b,
+                        ppi2=rep(NA,ns0*ng0), predRE=rep(NA,ns0*nea0),
+                        resid_m=rep(NA,nobs0), resid_ss=rep(NA,nobs0),
+                        marker=rep(NA,nsim), transfY=rep(NA,nsim),
+                        Yobs=rep(NA,nobs0), 
+                        pred_m_g=rep(NA,nobs0*ng0), pred_ss_g=rep(NA,nobs0*ng0),
+                        vraisdiscret=NA, UACV=NA, rlindiv=rep(NA,ns0),
+                        gconv=c(res$ca, res$cb, res$rdm), niter=res$ni,
+                        loglik=res$fn.value)
+        }            
 
+        if(out$conv %in% c(1,2,3))
+        {
+            estim0 <- 0
+            ll <- 0
+            ppi0 <- rep(0,ns0*ng0)
+            resid_m <- rep(0,nobs0)
+            resid_ss <- rep(0,nobs0)
+            pred_m_g <- rep(0,nobs0*ng0)
+            pred_ss_g <- rep(0,nobs0*ng0)
+            predRE <- rep(0,ns0*nea0)
+            marker <- rep(0,nsim)
+            transfY <- rep(0,nsim)
+            Yobs <- rep(0,nobs0)
+            vraisdiscret <- 0
+            UACV <- 0
+            rlindiv <- rep(0,ns0)
+            post <- .Fortran(C_logliklcmmcont,
+                             as.double(Y0),
+                             as.double(X0),
+                             as.integer(prior0),
+                             as.integer(idprob0),
+                             as.integer(idea0),
+                             as.integer(idg0),
+                             as.integer(idcor0),
+                             as.integer(ns0),
+                             as.integer(ng0),
+                             as.integer(nv0),
+                             as.integer(nobs0),
+                             as.integer(nea0),
+                             as.integer(nmes0),
+                             as.integer(idiag0),
+                             as.integer(nwg0),
+                             as.integer(ncor0),
+                             as.integer(NPM),
+                             as.double(out$best),
+                             ppi=as.double(ppi0),
+                             resid_m=as.double(resid_m),
+                             resid_ss=as.double(resid_ss),
+                             pred_m_g=as.double(pred_m_g),
+                             pred_ss_g=as.double(pred_ss_g),
+                             predRE=as.double(predRE),
+                             as.double(epsY),
+                             as.integer(idlink0),
+                             as.integer(nbzitr0),
+                             as.double(zitr),
+                             marker=as.double(marker),
+                             transfY=as.double(transfY),
+                             as.integer(nsim),
+                             Yobs=as.double(Yobs),
+                             Ydiscret=as.integer(Ydiscrete),
+                             vraisdiscret=as.double(vraisdiscret),
+                             UACV=as.double(UACV),
+                             rlindiv=as.double(rlindiv),
+                             as.double(out$V),
+                             as.integer(fix0),
+                             as.integer(nfix),
+                             as.double(bfix),
+                             as.integer(estim0),
+                             loglik=as.double(ll))
+            
+            out$ppi2 <- post$ppi
+            out$predRE <- post$predRE
+            out$Yobs <- post$Yobs
+            out$resid_m <- post$resid_m
+            out$resid_ss <- post$resid_ss
+            out$marker <- post$marker
+            out$transfY <- post$transfY
+            out$pred_m_g <- post$pred_m_g
+            out$pred_ss_g <- post$pred_ss_g
+            out$UACV <- post$UACV
+            out$rlindiv <- post$rlindiv
+            out$vraisdiscret <- post$vraisdiscret
+        }
+            
+        
 
+        ## creer best a partir de b et bfix
+        best <- rep(NA,length(fix0))
+        best[which(fix0==0)] <- out$best
+        best[which(fix0==1)] <- bfix
+        names(best) <- names_best
+        out$best <- best
+        NPM <- NPM+nfix
+        
 
 ### mettre NA pour les variances et covariances non calculees et  0 pr les prm fixes
-    if(length(posfix))
+        if(length(posfix))
         {
             if(out$conv==3)
-                {
-                    mr <- NPM-sum(pbH0)-length(posfix)
-                    Vr <- matrix(0,mr,mr)
-                    Vr[upper.tri(Vr,diag=TRUE)] <- out$V[1:(mr*(mr+1)/2)]
-                    Vr <- t(Vr)
-                    Vr[upper.tri(Vr,diag=TRUE)] <- out$V[1:(mr*(mr+1)/2)]
-                    V <- matrix(NA,NPM,NPM)
-                    V[setdiff(1:NPM,c(which(pbH0==1),posfix)),setdiff(1:NPM,c(which(pbH0==1),posfix))] <- Vr
-                    V[,posfix] <- 0
-                    V[posfix,] <- 0
-                    V <- V[upper.tri(V,diag=TRUE)]
-                }
+            {
+                mr <- NPM-sum(pbH0)-length(posfix)
+                Vr <- matrix(0,mr,mr)
+                Vr[upper.tri(Vr,diag=TRUE)] <- out$V[1:(mr*(mr+1)/2)]
+                Vr <- t(Vr)
+                Vr[upper.tri(Vr,diag=TRUE)] <- out$V[1:(mr*(mr+1)/2)]
+                V <- matrix(NA,NPM,NPM)
+                V[setdiff(1:NPM,c(which(pbH0==1),posfix)),setdiff(1:NPM,c(which(pbH0==1),posfix))] <- Vr
+                V[,posfix] <- 0
+                V[posfix,] <- 0
+                V <- V[upper.tri(V,diag=TRUE)]
+            }
             else
-                {
-                    mr <- NPM-length(posfix)
-                    Vr <- matrix(0,mr,mr)
-                    Vr[upper.tri(Vr,diag=TRUE)] <- out$V[1:(mr*(mr+1)/2)]
-                    Vr <- t(Vr)
-                    Vr[upper.tri(Vr,diag=TRUE)] <- out$V[1:(mr*(mr+1)/2)]
-                    V <- matrix(0,NPM,NPM)
-                    V[setdiff(1:NPM,posfix),setdiff(1:NPM,posfix)] <- Vr
-                    V <- V[upper.tri(V,diag=TRUE)]
-                }
+            {
+                mr <- NPM-length(posfix)
+                Vr <- matrix(0,mr,mr)
+                Vr[upper.tri(Vr,diag=TRUE)] <- out$V[1:(mr*(mr+1)/2)]
+                Vr <- t(Vr)
+                Vr[upper.tri(Vr,diag=TRUE)] <- out$V[1:(mr*(mr+1)/2)]
+                V <- matrix(0,NPM,NPM)
+                V[setdiff(1:NPM,posfix),setdiff(1:NPM,posfix)] <- Vr
+                V <- V[upper.tri(V,diag=TRUE)]
+            }
         }
-    else
+        else
         {
             if(out$conv==3)
-                {
-                    mr <- NPM-sum(pbH0)
-                    Vr <- matrix(0,mr,mr)
-                    Vr[upper.tri(Vr,diag=TRUE)] <- out$V[1:(mr*(mr+1)/2)]
-                    Vr <- t(Vr)
-                    Vr[upper.tri(Vr,diag=TRUE)] <- out$V[1:(mr*(mr+1)/2)]
-                    V <- matrix(NA,NPM,NPM)
-                    V[setdiff(1:NPM,which(pbH0==1)),setdiff(1:NPM,which(pbH0==1))] <- Vr
-                    V <- V[upper.tri(V,diag=TRUE)]
-                }
+            {
+                mr <- NPM-sum(pbH0)
+                Vr <- matrix(0,mr,mr)
+                Vr[upper.tri(Vr,diag=TRUE)] <- out$V[1:(mr*(mr+1)/2)]
+                Vr <- t(Vr)
+                Vr[upper.tri(Vr,diag=TRUE)] <- out$V[1:(mr*(mr+1)/2)]
+                V <- matrix(NA,NPM,NPM)
+                V[setdiff(1:NPM,which(pbH0==1)),setdiff(1:NPM,which(pbH0==1))] <- Vr
+                V <- V[upper.tri(V,diag=TRUE)]
+            }
             else
-                {
-                    V <- out$V
-                }
+            {
+                V <- out$V
+            }
         }
 
 
@@ -1059,7 +1048,6 @@
             colnames(pred)<-c(nom.subject,"pred_m","resid_m","pred_ss","resid_ss","obs",temp,temp1,var.time)
         }
         
-        names(out$best) <- names(b)
         
         estimlink <- cbind(out$marker,out$transfY)
         colnames(estimlink) <- c("Y","transfY")
@@ -1067,10 +1055,12 @@
 ### ad 2/04/2012
         if (!("intercept" %in% nom.X0)) X0.names2 <- X0.names2[-1]
 ### ad
-        res <-list(ns=ns0,ng=ng0,idea0=idea0,idprob0=idprob0,idg0=idg0,idcor0=idcor0,loglik=out$loglik,best=out$best,V=V,gconv=out$gconv,conv=out$conv,call=call,niter=out$niter,N=N,idiag=idiag0,pred=pred,pprob=ppi,predRE=predRE,Xnames=nom.X0,Xnames2=X0.names2,cholesky=Cholesky,estimlink=estimlink,epsY=epsY,linktype=idlink0,linknodes=zitr,Ydiscrete=Ydiscrete,discrete_loglik=out$vraisdiscret,UACV=out$UACV,IndivContrib=out$rlindiv,na.action=na.action,AIC=2*(length(out$best)-length(posfix)-out$loglik),BIC=(length(out$best)-length(posfix))*log(ns0)-2*out$loglik,data=datareturn,wRandom=wRandom,b0Random=b0Random, var.time=var.time)
+        cost <- proc.time()-ptm
+        
+        res <-list(ns=ns0,ng=ng0,idea0=idea0,idprob0=idprob0,idg0=idg0,idcor0=idcor0,loglik=out$loglik,best=out$best,V=V,gconv=out$gconv,conv=out$conv,call=call,niter=out$niter,N=N,idiag=idiag0,pred=pred,pprob=ppi,predRE=predRE,Xnames=nom.X0,Xnames2=X0.names2,cholesky=Cholesky,estimlink=estimlink,epsY=epsY,linktype=idlink0,linknodes=zitr,Ydiscrete=Ydiscrete,discrete_loglik=out$vraisdiscret,UACV=out$UACV,IndivContrib=out$rlindiv,na.action=na.action,AIC=2*(length(out$best)-length(posfix)-out$loglik),BIC=(length(out$best)-length(posfix))*log(ns0)-2*out$loglik,data=datareturn,wRandom=wRandom,b0Random=b0Random, var.time=var.time, runtime=cost[3])
         class(res) <- c("lcmm") 
 
-        cost <- proc.time()-ptm
         if(verbose==TRUE) cat("The program took", round(cost[3],2), "seconds \n") 
         res
     }
+
