@@ -1,4 +1,4 @@
-
+#' @export
 estimates.multlcmm <- function(x,cholesky=TRUE)
 {
  if(missing(x)) stop("The argument x should be specified")
@@ -34,5 +34,3 @@ estimates.multlcmm <- function(x,cholesky=TRUE)
  
  return(res)
 }
-
-estimates <- function(x,cholesky=TRUE) UseMethod("estimates")
